Index: drivers/dahdi/wcb4xxp/wcb4xxp.h
===================================================================
--- drivers/dahdi/wcb4xxp/wcb4xxp.h	(revision 5302)
+++ drivers/dahdi/wcb4xxp/wcb4xxp.h	(working copy)
@@ -450,6 +450,7 @@ struct b4xxp {
 	struct b4xxp_span spans[MAX_SPANS_PER_CARD];	/* Individual spans */
 	int order;				/* Order */
 	int flags;				/* Device flags */
+	int has_ec;				/* Has Echo Canceller? */
 	int master;				/* Are we master */
 	int ledreg;				/* copy of the LED Register */
 	unsigned int gpio;
Index: drivers/dahdi/wcb4xxp/base.c
===================================================================
--- drivers/dahdi/wcb4xxp/base.c	(revision 5302)
+++ drivers/dahdi/wcb4xxp/base.c	(working copy)
@@ -108,9 +108,12 @@ static struct proc_dir_entry *myproc;
 struct devtype {
 	char *desc;
 	unsigned int flags;
+	int ports; /* FIXME: Assumed to be 4 */
+	int has_ec;
 };
 
-static struct devtype wcb4xxp = { "Wildcard B410P", 0 };
+static struct devtype wcb4xxp = { "Wildcard B410P", .ports = 4, .has_ec = 1 };
+static struct devtype hfc4s =   { "HFC-4S PCI",     .ports = 4 };
 
 
 #if 0
@@ -603,6 +606,9 @@ static void ec_init(struct b4xxp *b4)
 	unsigned char b;
 	unsigned int i, j, mask;
 
+	if (! b4->has_ec)
+		return;
+
 /* Setup GPIO */
 	for (i=0; i < NUM_EC; i++) {
 		b = ec_read(b4, i, 0x1a0);
@@ -858,7 +864,8 @@ static void hfc_assign_bchan_fifo_ec(str
 
 	if (first) {
 		first = 0;
-		dev_info(b4->dev, "Hardware echo cancellation enabled.\n");
+		if (b4->has_ec)
+			dev_info(b4->dev, "Hardware echo cancellation enabled.\n");
 	}
 
 	fifo = port * 2;
@@ -1815,6 +1822,10 @@ static void b4xxp_update_leds(struct b4x
 	int i;
 	struct b4xxp_span *bspan;
 
+	/* FIXME: abusing has_ec as a marker for leds handling? */
+	if (!b4->has_ec)
+		return;
+
 	b4->blinktimer++;
 	for (i=0; i < b4->numspans; i++) {
 		bspan = &b4->spans[i];
@@ -2128,7 +2139,7 @@ static void init_spans(struct b4xxp *b4)
 		bspan->span.close  = b4xxp_close;
 		bspan->span.ioctl = b4xxp_ioctl;
 		bspan->span.hdlc_hard_xmit = b4xxp_hdlc_hard_xmit;
-		if (vpmsupport)
+		if (vpmsupport && b4->has_ec)
 			bspan->span.echocan = b4xxp_echocan;
 
 /* HDLC stuff */
@@ -2447,6 +2458,7 @@ static int __devinit b4xx_probe(struct p
 /* card found, enabled and main struct allocated.  Fill it out. */
 	b4->magic = WCB4XXP_MAGIC;
 	b4->variety = dt->desc;
+	b4->has_ec = dt->has_ec;
 
 	b4->pdev = pdev;
 	b4->dev = &pdev->dev;
@@ -2624,6 +2636,9 @@ static void __devexit b4xxp_remove(struc
 static struct pci_device_id b4xx_ids[] __devinitdata =
 {
 	{ 0xd161, 0xb410, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&wcb4xxp },
+	/* TODO: this is too generic. Replace with specific entries for 
+	 * specific devices using sub vendor IDs: */
+	{ 0x1397, 0x08b4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&hfc4s },
 	{ 0, }
 };
 
